@@ -33,7 +33,7 @@ enum x300_clock_which_t
     X300_CLOCK_WHICH_DB0_TX,
     X300_CLOCK_WHICH_DB1_RX,
     X300_CLOCK_WHICH_DB1_TX,
-    X300_CLOCK_WHICH_TEST,
+    X300_CLOCK_WHICH_FPGA,
 };
 
 class x300_clock_ctrl : boost::noncopyable
@@ -48,6 +48,7 @@ public:
             const size_t slaveno,
             const size_t hw_rev,
             const double master_clock_rate,
+            const double dboard_clock_rate,
             const double system_ref_rate);
 
     /*! Get the master clock rate of the device.
@@ -94,6 +95,22 @@ public:
      */
     virtual void set_ref_out(const bool) = 0;
 
+    /*! Set the clock delay for the given clock divider.
+     * \param which which clock
+     * \param rate the delay in nanoseconds
+     * \param resync resync clocks to apply delays
+     * \return the actual delay value set
+     * \throw exception when which invalid or delay_ns out of range
+     */
+    virtual double set_clock_delay(const x300_clock_which_t which, const double delay_ns, const bool resync = true) = 0;
+
+    /*! Get the clock delay for the given clock divider.
+     * \param which which clock
+     * \return the actual delay value set
+     * \throw exception when which invalid
+     */
+    virtual double get_clock_delay(const x300_clock_which_t which) = 0;
+
     /*! Reset the clocks.
      *  Should be called if the reference clock changes
      *  to reduce the time required to achieve a lock.
